import React, { Component } from 'react';
import { connect } from 'react-redux';
import { t, tu } from '../../utils/i18n';
import { Client } from '../../services/api';
import { Modal, ModalBody, ModalHeader } from 'reactstrap';
import SweetAlert from 'react-bootstrap-sweetalert';
import { WITNESS_CREATE_COST } from '../../constants';
import { FormattedNumber } from 'react-intl';
import { transactionResultManager } from '../../utils/mcash';
import { prettyErrorMessage } from '../../utils/helper';

const {isWebUri} = require('valid-url');

class ApplyForDelegate extends Component {

  constructor () {
    super();
    this.state = {
      url: '',
      check: false,
      applyResponse: null,
      loading: false,
      modal: null,
      witnessAddress: ''
    };
  }

  isValid = () => {
    const {check, witnessAddress} = this.state;
    return check && !!witnessAddress && this.isValidUrl();
  };

  isValidUrl = () => {
    const {url} = this.state;
    return url.length > 0 && isWebUri(url);
  };

  setWitnessAddress = (address) => {
    this.setState({
      witnessAddress: address
    })
  }

  doApply = async () => {
    let res;
    let errorMessage = '';
    let {account} = this.props;
    let {url, witnessAddress} = this.state;
    this.setState({loading: true});


    if (account && (account.type === 'ACCOUNT_LEDGER' || account.type === 'ACCOUNT_MCASHLINK' )) {
      let mcashWeb;
      let ownerAddress;
      if (this.props.walletType.type === 'ACCOUNT_LEDGER') {
        mcashWeb = this.props.mcashWeb();
        ownerAddress = account.address;
      } else if (this.props.walletType.type === 'ACCOUNT_MCASHLINK') {
        mcashWeb = account.mcashWeb;
        ownerAddress = mcashWeb.defaultAddress.hex;
      }
      try {
        const unSignTransaction = await mcashWeb.transactionBuilder.applyForSR(witnessAddress,ownerAddress, url);
        const {result} = await transactionResultManager(unSignTransaction, mcashWeb);
        res = result;
      } catch (e) {
        if (typeof e === 'string') {
          errorMessage = e;
        }
      }
    } else {
      let {success} = await Client.applyForDelegate(witnessAddress,account.address,url)(account.key);
      res = success;
    }
    this.setState({loading: false});
    if (res) {
      this.confirm();
    } else {
      errorMessage = prettyErrorMessage(errorMessage);
      this.setState({
        modal: (
          <SweetAlert warning title={tu('error')} onConfirm={this.hideModal}>
            {tu('apply_representative_error_message_0')} <br/>
            {errorMessage ? errorMessage : tu('apply_representative_error_message_1')}
          </SweetAlert>
        )
      })
    }
  };

  hideModal = () => {
    this.setState({
      modal: null,
    });
  };

  confirm = () => {
    let {onConfirm} = this.props;
    onConfirm && onConfirm();
  };

  cancel = () => {
    let {onCancel} = this.props;
    onCancel && onCancel();
  };

  render () {
    const {check, loading, modal, url, witnessAddress} = this.state;

    if (modal) {
      return modal;
    }

    return (
      <Modal isOpen={true} toggle={this.cancel} fade={false} size="lg" className="modal-dialog-centered">
        <ModalHeader className="text-center _applyHeader"
                     toggle={this.cancel}>{tu('Super Nodes')}</ModalHeader>
        <ModalBody>
          <p className="card-text text-left _applyDesc">
            {t('apply_for_delegate_description')}
          </p>
          <hr/>
          <div className="text-left mb-1">
            <label className="_applyTitle">
              <span>{'Your witness address'}</span>
            </label>
            <input type="text"
                   className="form-control"
                   style={{borderRadius: '0px', background: '#F3F3F3', border: '1px solid #EEEEEE '}}
                   value={witnessAddress}
                   onChange={(ev) => this.setWitnessAddress(ev.target.value)}/>
          </div>
          <div className="text-left mb-1">
            <label className="_applyTitle">{tu('your_personal_website_address')}</label>
            <input className={'form-control text-left ' + (check && !this.isValidUrl() ? ' is-invalid' : '')}
                   type="text"
                   placeholder="https://"
                   style={{borderRadius: '0px', background: '#F3F3F3', border: '1px solid #EEEEEE '}}
                   value={url}
                   onChange={(ev) => this.setState({url: ev.target.value})}/>
            <div className="invalid-feedback text-left text-danger">
              {tu('invalid_url')}
            </div>
          </div>
          <div>
            <div className="form-check">
              <input id="chkConfirm"
                     type="checkbox"
                     className="form-check-input"
                     checked={check}
                     onChange={(ev) => this.setState({check: ev.target.checked})}/>
              <label htmlFor="chkConfirm" className="form-check-label _apply">
                {tu('understand_tron_sr_message_0')}
                <b> <FormattedNumber value={WITNESS_CREATE_COST}/> MCASH</b> {t('understand_tron_sr_message_1')}
              </label>
            </div>
          </div>
          <div className="pt-3">
            <p className="text-center">
              <button
                disabled={!this.isValid() || loading}
                className="btn btn-success"
                style={{width: '220px', background: '#69C265'}}
                onClick={this.doApply}>{tu('submit')}</button>
            </p>
          </div>
        </ModalBody>
      </Modal>
    )
  }
}

function mapStateToProps (state) {
  return {
    account: state.app.account,
    walletType: state.app.wallet,
  };
}

const mapDispatchToProps = {};

export default connect(mapStateToProps, mapDispatchToProps)(ApplyForDelegate)
