/* eslint-disable no-restricted-globals */
import { connect } from 'react-redux';
import { FormattedNumber, injectIntl } from 'react-intl';
import React from 'react';
import { Modal, ModalBody, ModalHeader } from 'reactstrap';
import { Client } from '../../services/api';
import { ONE_UNIT } from '../../constants';
import { reloadWallet } from '../../actions/wallet';
import { NumberField } from '../common/Fields';
import { transactionResultManager } from '../../utils/mcash';
import Lockr from 'lockr';
import { withTronWeb } from '../../utils/mcashWeb';

const perStaking = 5000

@connect(
  state => ({
    account: state.app.account,
    wallet: state.app.wallet,
    tokenBalances: state.account.tokens,
    accountReducer: state.account,
    currentWallet: state.wallet.current
  }),
  {
    reloadWallet
  }
)
@injectIntl
@withTronWeb
export default class StakeModal extends React.PureComponent {

  constructor (props) {
    super(props);
    this.state = {
      loading: false,
      confirmed: false,
      amount: ''
    };
  }

  componentDidMount () {
    this.props.reloadWallet();
  }

  hideModal = () => {
    let {onHide} = this.props;
    onHide && onHide();
  };

  confirmModal = () => {
    let {onConfirm} = this.props;
    let {amount} = this.state;
    onConfirm && onConfirm({
      amount
    });
  };

  onAmountChanged = (value) => {
    // let {mcashBalance} = this.props;
    let amount = parseInt(value);
    if (!isNaN(amount)) {
      amount = amount > 0 ? Math.floor(amount) : Math.abs(amount);
      // amount = amount < mcashBalance ? amount : mcashBalance;
    } else {
      amount = '';
    }

    this.setState({
      amount,
    });
  };

  setInputMax = () => {
    let {accountReducer, currentWallet} = this.props;
    const mcashBalance = (accountReducer && accountReducer.mcashBalance) || (currentWallet && currentWallet.mcashBalance) || 0;
    let value = Math.floor(mcashBalance) - (Math.floor(mcashBalance) % perStaking)
    this.setState({
      amount: Math.floor(value)
    })
  }

  stake = async () => {
    let {account, onError, wallet} = this.props;
    let {amount, receiver, confirmed} = this.state;
    if (!confirmed && amount > 0 && amount % perStaking === 0) {
      return;
    }
    let res, result;
    this.setState({loading: true});

    try {
      if (Lockr.get('islogin') || this.props.wallet.type === 'ACCOUNT_LEDGER' || this.props.wallet.type === 'ACCOUNT_MCASHLINK') {
        const tronWebLedger = this.props.mcashWeb();
        const {mcashWeb} = this.props.account;

        if (this.props.wallet.type === 'ACCOUNT_LEDGER') {
          let unSignTransaction;
          if (receiver === '') {
            unSignTransaction = await tronWebLedger.transactionBuilder.stake(amount * ONE_UNIT, 3);
          } else {
            unSignTransaction = await tronWebLedger.transactionBuilder.stake(amount * ONE_UNIT, 3, wallet.address);
          }
          result = await transactionResultManager(unSignTransaction, tronWebLedger);
        }
        if (this.props.wallet.type === 'ACCOUNT_MCASHLINK') {
          let unSignTransaction = await mcashWeb.transactionBuilder.stake(amount * ONE_UNIT, 3, mcashWeb.defaultAddress.base58).catch(e => false);
          result = await transactionResultManager(unSignTransaction, mcashWeb)
        }

        res = result;
      } else {
        let {success} = await Client.stake(account.address, amount * ONE_UNIT, 3)(account.key);
        res = success
      }

      if (res) {
        this.setState({
          loading: false
        }, () => {
          this.confirmModal({amount});
        });
      } else {
        throw new Error('Failed to stake');
      }

    } catch (e) {
      console.error(e);
      onError && onError();
    }

  };

  render () {
    let {amount, confirmed} = this.state;
    let {accountReducer, currentWallet} = this.props;
    const mcashBalance = (accountReducer && accountReducer.mcashBalance) || (currentWallet && currentWallet.mcashBalance) || 0;
    let isValid = (amount > 0 && amount % perStaking === 0 && mcashBalance >= amount && confirmed);

    return (
      <Modal
        isOpen={true}
        toggle={this.hideModal}
        fade={false}
        className="modal-dialog-centered _freezeContent"
      >
        <ModalHeader className="text-center _freezeHeader" toggle={this.hideModal}>
          <span>{ 'Stake' }</span>
        </ModalHeader>

        <ModalBody className="text-center _freezeBody">
          <div className="form-group">
            <div style={{position: 'relative'}}>
              <button
                type="button"
                onClick={this.setInputMax}
                style={styles.maxButton}
              >{'MAX'}</button>
              <NumberField
                min={1}
                decimals={0}
                value={amount}
                placeholder={'MCASH Amount (Divisible by 5000)'}
                className="form-control text-left"
                style={{marginTop: '12px', background: '#F3F3F3', border: '1px solid #EEEEEE'}}
                onChange={this.onAmountChanged}
              />
              {
                (amount % perStaking !== 0) && (
                  <div className='text-left' style={{ color: 'red' }}>
                    <small>{ `Invalid amount (Divisible by ${perStaking})` }</small>
                  </div>
                )
              }
            </div>
          </div>

          <div className="form-check">
            <input
              id="checkboxConfirm"
              type="checkbox"
              className="form-check-input"
              onChange={(ev) => this.setState({confirmed: ev.target.checked})}/>
            <label htmlFor="checkboxConfirm" className="form-check-label _freeze">
              { 'I confirm to stake ' }
              <b><FormattedNumber value={amount}/> MCASH</b>
            </label>
          </div>

          <p className="mt-3">
            <button className="btn btn-primary col-sm"
                    disabled={!isValid}
                    onClick={this.stake}
                    style={{background: '#4A90E2', borderRadius: '0px', border: '0px'}}
            >
              <i className="fa fa-users mr-2"/>
              <span>{ 'Stake' }</span>
            </button>
          </p>
        </ModalBody>
      </Modal>
    )
  }
}

const styles = {
  maxButton: {
    position: 'absolute',
    right: 0,
    top: 0,
    background: 'none',
    height: '35px',
    border: 'none',
    cursor: 'pointer',
  }
};
