import React, { Fragment } from 'react';
import { Client } from '../../../services/api';
import { AddressLink, TransactionHashLink, TokenTRC20Link } from '../../common/Links';
import { tu } from '../../../utils/i18n';
import TimeAgo from 'react-timeago';
import moment from 'moment';
import { Truncate } from '../../common/text';
import { withTimers } from '../../../utils/timing';
import { injectIntl } from 'react-intl';
import SmartTable from '../../common/SmartTable.js'
import { upperFirst } from 'lodash';
import { TronLoader } from '../../common/loaders';
import TotalInfo from '../../common/TableTotal';
// import DateRange from '../../common/DateRange';
import { FormatNumberByDecimals } from '../../../utils/number'
import { mapContractTokenData } from '../../../utils/helper';

class Transfers extends React.Component {
  _mounted = false

  constructor (props) {
    super(props);
    this.start = moment([2018, 5, 25]).startOf('day').valueOf();
    this.end = moment().valueOf();
    this.state = {
      filter: {},
      transfers: [],
      page: 0,
      total: 0,
      pageSize: 25,
      showTotal: props.showTotal !== false,
      emptyState: props.emptyState,
      autoRefresh: props.autoRefresh || false
    };
  }

  componentDidMount () {
    this._mounted = true
    this.loadPage();

    if (this.state.autoRefresh !== false) {
      this.props.setInterval(() => this.load(), this.state.autoRefresh);
    }
  }

  componentWillUnmount() {
    this._mounted = false
  }

  onChange = (page, pageSize) => {
    this.loadPage(page, pageSize);
  };

  loadPage = async (page = 1, pageSize = 20) => {
    let {filter} = this.props;

    this.setState({
      loading: true,
      page: page,
      pageSize: pageSize,
    });

    const result = await Client.getToken20Transfers({
      limit: pageSize,
      start: (page - 1) * pageSize,
      contract_address: filter.token
    });
    let transfers = mapContractTokenData(result);
    if (Array.isArray(transfers)) {
      transfers.forEach((item, index) => {
        item.index = index + 1;
      })
    }
    if (!this._mounted) return

    this.setState({
      page,
      transfers,
      total: result.total,
      rangeTotal: result.range_total,
      loading: false,
    });
  };

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: upperFirst(intl.formatMessage({id: 'hash'})),
        dataIndex: 'transaction_hash',
        key: 'transaction_hash',
        className: 'ant_table',
        render: (text) => {
          return (
            <Truncate>
              <TransactionHashLink hash={text}>
                {text}
              </TransactionHashLink>
            </Truncate>
          )
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'age'})),
        dataIndex: 'timestamp',
        key: 'timestamp',
        width: '150px',
        className: 'ant_table',
        render: (text) => {
          return <TimeAgo date={Number(text)}
                          title={moment(text).format('MMM-DD-YYYY HH:mm:ss A')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'from'})),
        dataIndex: 'from_address',
        key: 'from_address',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text}/>
        }
      },
      {
        title: '',
        className: 'ant_table',
        width: '30px',
        render: () => {
          return <img src={require('../../../images/arrow.png')}/>
        },
      },
      {
        title: upperFirst(intl.formatMessage({id: 'to'})),
        dataIndex: 'to_address',
        key: 'to_address',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text}/>
        },
      },
      {
        title: upperFirst(intl.formatMessage({id: 'amount'})),
        dataIndex: 'amount',
        key: 'amount',
        width: '200px',
        align: 'right',
        className: 'ant_table',
        render: (text, record) => {
          const token = record.token ? record.token : {}
          return (
            <span>
              {/*<FormattedNumber value={parseFloat(record.quant) / (Math.pow(10,token.decimals))}/>*/}
              <span>{FormatNumberByDecimals(record.amount, token.decimals)}</span>
              &nbsp;&nbsp;
              <TokenTRC20Link name={token.symbol} address={token.contract_address}/>
            </span>
          )
        },
      }
    ];
  }

  // onDateOk (start, end) {
  //   this.start = start.valueOf();
  //   this.end = end.valueOf();
  //   let {page, pageSize} = this.state;
  //   this.loadPage(page, pageSize);
  // }

  render () {
    // let {transfers, total, rangeTotal, loading, emptyState: EmptyState = null} = this.state;
    let {transfers, total, rangeTotal, loading} = this.state;
    if (total === 0) {
      transfers = [];
    }
    // let {intl} = this.props;
    let column = this.customizedColumn();
    // let tableInfo = intl.formatMessage({id: 'a_totle'}) + ' ' + total + ' ' + intl.formatMessage({id: 'transaction_info'});
    // if (!loading && transfers.length === 0) {
    //     if (!EmptyState) {
    //         return (
    //             <div className="p-3 text-center no-data">{tu("no_transfers")}</div>
    //         );
    //     }
    //
    //     return <EmptyState/>;
    // }

    return (
      <Fragment>
        {loading && <div className="loading-style" style={{marginTop: '-20px'}}><TronLoader/></div>}
        <div className="row transfers">
          <div className="col-md-12 table_pos">
            <div className="d-flex justify-content-between pl-3 pr-3" style={{left: 'auto'}}>
              {total ? <TotalInfo total={total} rangeTotal={rangeTotal} typeText="transaction_info"
                                  divClass="table_pos_info_addr"/> : ''}
              {/*<DateRange onDateOk={(start, end) => this.onDateOk(start, end)} dateClass="date-range-box-TRC20token"/>*/}
            </div>
            {
              (!loading && transfers.length === 0) ?
                <div className="pt-5 pb-5 text-center no-data transfers-bg-white">{tu('no_transfers')}</div>
                : <SmartTable border={false} loading={loading} column={column} data={transfers} total={total}
                              addr="address" transfers="token"
                              onPageChange={(page, pageSize) => {
                                this.loadPage(page, pageSize)
                              }}/>
            }
          </div>
        </div>
      </Fragment>
    )
  }
}

export default withTimers(injectIntl(Transfers));
