import React, {Fragment} from 'react';
import {Client} from '../../../services/api';
import {TransactionHashLink, BlockNumberLink, AddressLink} from '../../common/Links';
import {injectIntl} from 'react-intl';
import {tu} from '../../../utils/i18n';
import TimeAgo from 'react-timeago';
import {TronLoader} from '../../common/loaders';
import {Truncate} from '../../common/text';
import SmartTable from '../../common/SmartTable.js'
import {upperFirst} from 'lodash';
import {TRXPrice} from '../../common/Price';
import TotalInfo from '../../common/TableTotal';
import {Tooltip} from 'antd'
import moment from 'moment';
import {fromAmount, isTriggerSmartContract} from '../../../utils/helper';

class Transactions extends React.Component {
  _mounted = false

  constructor(props) {
    super(props);
    this.start = moment([2018, 5, 25]).startOf('day').valueOf();
    this.end = moment().valueOf();
    this.state = {
      filter: {},
      transactions: [],
      total: 0,
      emptyState: props.EmptyState || (
        <TronLoader>
          Loading Transactions
        </TronLoader>
      )
    };
  }

  componentDidMount() {
    this._mounted = true
    return this.loadTransactions();
  }

  componentWillUnmount() {
    this._mounted = false
  }

  onChange = (page, pageSize) => {
    return this.loadTransactions(page, pageSize);
  };

  loadTransactions = async (page = 1, pageSize = 20) => {
    let {filter} = this.props;
    this.setState({
      loading: true,
      page: page,
      pageSize: pageSize,
    });

    let transactions = await Client.getTransactions({
      limit: pageSize,
      start: (page - 1) * pageSize,
      ...filter,
    });
    if (transactions && this._mounted) {
      transactions.items.forEach(item => {
        item.tip = item.ownerAddress === filter.contract ? 'out' : 'in'
      });
      this.setState({
        transactions: transactions.items,
        total: transactions.total,
        rangeTotal: transactions.rangeTotal,
        loading: false
      });
    }
  };

  customizedColumn = () => {
    let {intl, isInternal = false} = this.props;

    return [
      {
        title: upperFirst(intl.formatMessage({id: isInternal ? 'ParentTxHash' : 'TxHash'})),
        dataIndex: 'hash',
        key: 'hash',
        align: 'left',
        // width: '150px',
        className: 'ant_table',
        render: (text) => {
          return (
            <Truncate>
              <TransactionHashLink hash={text}>{text}</TransactionHashLink>
            </Truncate>
          )
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'block'})),
        dataIndex: 'block',
        key: 'block',
        align: 'left',
        width: '90px',
        className: 'ant_table',
        render: (text) => {
          return <BlockNumberLink number={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'age'})),
        dataIndex: 'timestamp',
        key: 'timestamp',
        align: 'left',
        width: '150px',
        className: 'ant_table',
        render: (text) => {
          return <TimeAgo date={text} title={moment(text).format('MMM-DD-YYYY HH:mm:ss A')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'from'})),
        dataIndex: 'owner_address',
        key: 'owner_address',
        align: 'left',
        render: (text, record) => {
          return record.tip === 'in' ? (
            <span className="d-flex">
              {/*{*/}
              {/*record.ownAddressType === 2 && (*/}
              {/*<Tooltip placement="top" title={intl.formatMessage({id: 'contracts'})}>*/}
              {/*<span><i className="far fa-file mr-1" /></span>*/}
              {/*</Tooltip>*/}
              {/*)*/}
              {/*}*/}
              {/*<AddressLink address={text} isContract={record.ownAddressType === 2}/>*/}
              <AddressLink address={text}/>
            </span>
          ) : (<Truncate><span>{text}</span></Truncate>)
        }
      },
      // {
      //   title: '',
      //   dataIndex: 'tip',
      //   align: 'center',
      //   render: (text, record, index) => {
      //     return <div className={"tip tip-"+text}>{toUpper(text)}</div>
      //   }
      // },
      {
        title: upperFirst(intl.formatMessage({id: 'to'})),
        dataIndex: 'to_address',
        key: 'to_address',
        align: 'left',
        render: (text, record) => {
          return record.tip === 'out' ? (
            <span className="d-flex">
                {/*{*/}
              {/*record.toAddressType === 2 && (*/}
              {/*<Tooltip placement="top" title={intl.formatMessage({id: 'contracts'})}>*/}
              {/*<span><i className="far fa-file mr-1"/></span>*/}
              {/*</Tooltip>*/}
              {/*)*/}
              {/*}*/}
              {/*<AddressLink address={text} isContract={record.toAddressType === 2}/>*/}
              {
                isTriggerSmartContract(record.contract_type) && (
                  <Tooltip placement="top" title={intl.formatMessage({id: 'contracts'})}>
                    <span><i className="far fa-file mr-1"/></span>
                  </Tooltip>
                )
              }
              <AddressLink address={text} isContract={isTriggerSmartContract(record.contract_type)}/>
              </span>
          ) : (<Truncate><span>{text}</span></Truncate>)
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'value'})),
        dataIndex: 'amount',
        key: 'amount',
        align: 'right',
        width: '120px',
        className: 'ant_table',
        render: (text) => {
          return <TRXPrice amount={fromAmount(text)}/>
        }
      },
      // {
      //   title: upperFirst(intl.formatMessage({id: 'TxFee'})),
      //   dataIndex: 'txFee',
      //   key: 'txFee',
      //   align: 'right',
      //   width: '120px',
      //   className: 'ant_table',
      //   render: (text, record, index) => {
      //     return  <TRXPrice amount={parseInt(text) / ONE_UNIT}/>
      //   }
      // }
    ];
  };


  render() {
    let {transactions, total, rangeTotal, loading} = this.state;
    let column = this.customizedColumn();

    return (
      <Fragment>
        {loading && <div className="loading-style" style={{marginTop: '-20px'}}><TronLoader/></div>}
        <div className="row">
          <div className="col-md-12 table_pos mt-5">
            {total ? <TotalInfo total={total} rangeTotal={rangeTotal} typeText="transactions_unit"/> : ''}
            {
              (!loading && transactions.length === 0) ? <div className="p-3 text-center no-data">{tu('no_tnx')}</div> :
                <SmartTable bordered={true} loading={loading}
                            column={column} data={transactions} total={total}
                            onPageChange={(page, pageSize) => {
                              return this.loadTransactions(page, pageSize)
                            }}/>
            }

          </div>
        </div>
      </Fragment>
    )
  }
}

export default injectIntl(Transactions)
