/* eslint-disable no-undef */
import React from 'react';
import {connect} from 'react-redux';
import {Route, Switch} from 'react-router-dom';
import {Client} from '../../../services/api';
import {tu} from '../../../utils/i18n';
import {FormattedDate, FormattedTime} from 'react-intl';
import {BlockNumberLink} from '../../common/Links';
import {CopyText} from '../../common/Copy';
import {TronLoader} from '../../common/loaders';
import {Truncate} from '../../common/text';
import Contract from '../../tools/TransactionViewer/Contract';
import {Alert} from 'reactstrap';
import {setLanguage} from '../../../actions/app'
import queryString from 'query-string';
import {mapTokenData, mapTokenObject} from '../../../utils/helper';

class Transaction extends React.Component {

  constructor() {
    super();
    this.state = {
      loading: true,
      notFound: false,
      transaction: {
        hash: -1,
        timestamp: 0,
      },
      tabs: {
        contracts: {
          id: 'contracts',
          icon: 'fa fa-exchange-alt',
          path: '',
          label: <span>{tu('contracts')}</span>,
          cmp: () => <TronLoader/>,
        }
      },
    };
  }

  componentDidMount() {
    document.documentElement.scrollTop = 0;
    let {match, location} = this.props;

    let lang = queryString.parse(location.search).lang;
    if (lang) {
      this.props.setLanguage(lang);
    }
    this.load(match.params.hash);
  }

  componentDidUpdate(prevProps) {
    let {match} = this.props;
    if (match.params.hash !== prevProps.match.params.hash) {
      this.load(match.params.hash);
    }
  }

  async load(id) {
    this.setState({loading: true, transaction: {hash: id}});

    const result = await Client.getTransactionByHash(id);
    if (!result['hash']) {
      this.setState({
        notFound: true,
      });
      return;
    }
    let tokenMap = result['token_map'];
    const transaction = mapTokenObject(result, tokenMap, 'asset_id');
    const internalTransactions = mapTokenData(transaction['internal_transactions'], tokenMap, 'asset_id');
    const tokenTransfers = mapTokenData(transaction['token_transactions'], tokenMap);

    this.setState({
      loading: false,
      transaction,
      tabs: {
        contracts: {
          id: 'contracts',
          icon: 'fa fa-exchange-alt',
          path: '',
          label: <span>{tu('contracts')}</span>,
          cmp: () => (
            <Contract contract={{
              ...transaction,
              ...(transaction['contract_data'] || {}),
              ...(transaction['trigger_info'] || {}),
              cost: transaction.cost,
              internalTransactions,
              tokenTransferInfo: tokenTransfers,
              contract_note: transaction.data,
              contractType: transaction.contract_type,
            }}/>
          ),
        },
      }
    });
  }

  render() {
    let {transaction, tabs, loading, notFound} = this.state;
    let {match} = this.props;
    if (notFound) {
      return (
        <main className="container header-overlap">
          <Alert color="warning" className="text-center">
            {tu('transaction_not_found')}
          </Alert>
        </main>
      );
    }
    return (
      <main className="container header-overlap">
        {
          loading ? (
              <div className="card">
                <TronLoader>
                  {tu('loading_transaction')}
                </TronLoader>
              </div>
            ) :
            <div className="row">
              <div className="col-md-12">
                <div className="card  list-style-header">
                  <div className="card-body">
                    <h5 className="card-title m-0">
                      <i className="fa fa-hashtag mr-1"/>
                      {tu('hash')} {transaction.hash}
                    </h5>
                  </div>
                  <div className="table-responsive">
                    <table className="table table-hover m-0">
                      <tbody>
                      {
                        transaction.hasOwnProperty('confirmed') && <tr>
                          <th>{tu('status')}:</th>
                          <td>
                            {
                              transaction.confirmed ?
                                <span className="badge badge-success text-uppercase">{tu('Confirmed')}</span> :
                                <span className="badge badge-danger text-uppercase">{tu('Unconfirmed')}</span>
                            }
                          </td>
                        </tr>
                      }
                      {
                        transaction.hasOwnProperty('contract_address') && <tr>
                          <th>{tu('result')}:</th>
                          <td>{transaction['contract_address']}</td>
                        </tr>
                      }

                      <tr>
                        <th>{tu('hash')}:</th>
                        <td>
                          <Truncate>
                            {transaction.hash}
                            <CopyText text={transaction.hash} className="ml-1"/>
                          </Truncate>
                        </td>
                      </tr>
                      <tr>
                        <th>{tu('block')}:</th>
                        <td><BlockNumberLink number={transaction['block_number']}/></td>
                      </tr>
                      {
                        transaction.timestamp !== 0 && <tr>
                          <th>{tu('time')}:</th>
                          <td>
                            <FormattedDate value={transaction.timestamp}/>&nbsp;
                            <FormattedTime value={transaction.timestamp}
                                           hour='numeric'
                                           minute="numeric"
                                           second='numeric'
                                           hour12={false}
                            />&nbsp;
                            {/*(<TimeAgoI18N date={transaction.timestamp} activeLanguage={activeLanguage}/>)*/}
                          </td>
                        </tr>
                      }
                      </tbody>
                    </table>
                  </div>
                </div>

                <div className="card mt-3  list-style-body">
                  <div className="card-body p-0  list-style-body__body" style={{'overflow': 'auto'}}>
                    <Switch>
                      {
                        Object.values(tabs).map(tab => (
                          <Route key={tab.id} exact path={match.url + tab.path} render={() => (<tab.cmp/>)}/>
                        ))
                      }
                    </Switch>
                  </div>
                </div>
              </div>
            </div>

        }
      </main>
    )
  }

}


function mapStateToProps() {
  return {};
}

const mapDispatchToProps = {
  setLanguage,
};

export default connect(mapStateToProps, mapDispatchToProps)(Transaction);
