import React from 'react';
import {injectIntl} from 'react-intl';
import {loadTokens} from '../../actions/tokens';
import {connect} from 'react-redux';
import TimeAgo from 'react-timeago';
import {Client} from '../../services/api';
import {AddressLink, BlockNumberLink, TransactionHashLink} from '../common/Links';
import {getQueryParams} from '../../utils/url';
import {Truncate} from '../common/text';
import {upperFirst} from 'lodash';
import SmartTable from '../common/SmartTable.js'
import {TronLoader} from '../common/loaders';
import TotalInfo from '../common/TableTotal';
import moment from 'moment';
import {NameWithId} from '../common/names';
import {mapAssetTokenData, mapTokenData} from '../../utils/helper';

class Transfers extends React.Component {

  constructor() {
    super();
    this.state = {
      transfers: [],
      total: 0,
    };
  }

  componentDidMount() {
    this.load();
  }

  onChange = (page, pageSize) => {
    this.load(page, pageSize);
  };

  load = async (page = 1, pageSize = 20) => {
    let {location} = this.props;
    this.setState(
      {
        loading: true,
        page: page,
        pageSize: pageSize,
      }
    );
    let searchParams = {};

    for (let [key, value] of Object.entries(getQueryParams(location))) {
      switch (key) {
        case 'address':
        case 'block_number':
          searchParams[key] = value;
          break;
        default:
          break;
      }
    }

    const result = await Client.getTransfers({
      limit: pageSize,
      start: (page - 1) * pageSize,
      ...searchParams,
    });
    let transfers = mapAssetTokenData(result);
    console.log(transfers);
    this.setState({
      transfers: transfers,
      loading: false,
      total: result.total,
      rangeTotal: result['range_total'],
    });
  };

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: '#',
        dataIndex: 'hash',
        key: 'hash',
        align: 'left',
        className: 'ant_table',
        width: '15%',
        render: (text) => {
          return <Truncate>
            <TransactionHashLink hash={text}>{text}</TransactionHashLink>
          </Truncate>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'block'})),
        dataIndex: 'block_number',
        key: 'block_number',
        align: 'left',
        className: 'ant_table',
        width: '10%',
        render: (text) => {
          return <BlockNumberLink number={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'created'})),
        dataIndex: 'timestamp',
        key: 'timestamp',
        align: 'left',
        width: '14%',
        render: (text) => {
          return <TimeAgo date={text} title={moment(text).format('MMM-DD-YYYY HH:mm:ss A')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'from'})),
        dataIndex: 'owner_address',
        key: 'owner_address',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'to'})),
        dataIndex: 'to_address',
        key: 'to_address',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'value'})),
        dataIndex: 'amount',
        key: 'amount',
        align: 'right',
        width: '180px',
        className: 'ant_table',
        render: (text, record) => {
          return <NameWithId value={record} type="symbol" totoken valueRange/>
        }
      },
      // {
      //     title: upperFirst(intl.formatMessage({id: 'status'})),
      //     dataIndex: 'confirmed',
      //     key: 'confirmed',
      //     align: 'center',
      //     className: 'ant_table',
      //     render: (text, record, index) => {
      //         return record.confirmed?
      //             <span className="badge badge-success text-uppercase">{intl.formatMessage({id:'Confirmed'})}</span> :
      //             <span className="badge badge-danger text-uppercase">{intl.formatMessage({id: 'Unconfirmed'})}</span>
      //     },
      // }
    ];
  }

  onDateOk(start, end) {
    this.start = start.valueOf();
    this.end = end.valueOf();
    let {page, pageSize} = this.state;
    this.load(page, pageSize);
  }

  render() {
    let {transfers, total, rangeTotal, loading} = this.state;
    // let {match, intl} = this.props;
    let column = this.customizedColumn();

    return (
      <main className="container header-overlap pb-3 token_black">
        {loading && <div className="loading-style"><TronLoader/></div>}
        <div className="row">
          <div className="col-md-12 table_pos">
            {total ? <TotalInfo total={total} rangeTotal={rangeTotal} typeText="transfers_unit"/> : ''}
            <SmartTable bordered={true} loading={loading} column={column} data={transfers} total={total}
                        onPageChange={(page, pageSize) => {
                          this.load(page, pageSize)
                        }}/>
          </div>
        </div>
      </main>
    )
  }
}

function mapStateToProps(state) {
  return {};
}

const mapDispatchToProps = {
  loadTokens,
};

export default connect(mapStateToProps, mapDispatchToProps)(injectIntl(Transfers));
