import React from 'react';
import { Client } from '../../services/api';
import { BlockNumberLink } from './Links';
import { t } from '../../utils/i18n';
import { FormattedNumber, injectIntl } from 'react-intl';
import TimeAgo from 'react-timeago';
import moment from 'moment';
import SmartTable from './SmartTable.js'
import { upperFirst } from 'lodash';
import { TronLoader } from './loaders';
import { CHAIN_SYMBOL, ONE_UNIT } from '../../constants';

class Blocks extends React.Component {

  constructor (props) {
    super(props);
    this.state = {
      filter: {},
      blocks: [],
      page: 0,
      total: 0,
      pageSize: 25,
      totalVotes: 0,
      emptyState: props.emptyState,
    };
  }

  componentDidMount () {
    this.load();
  }

  onChange = (page, pageSize) => {
    this.load(page, pageSize);
  };

  load = async (page = 1, pageSize = 20) => {
    let {filter} = this.props;
    this.setState({loading: true});

    let data = await Client.getBlocks({
      limit: pageSize,
      start: (page - 1) * pageSize,
      ...filter,
    });

    this.setState({
      page,
      blocks: data.items,
      total: data.total,
      loading: false,
    });
  };

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: upperFirst(intl.formatMessage({id: 'height'})),
        dataIndex: 'number',
        key: 'number',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <BlockNumberLink number={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'age'})),
        dataIndex: 'timestamp',
        key: 'timestamp',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <TimeAgo date={text} title={moment(text).format('MMM-DD-YYYY HH:mm:ss A')}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'transaction'})),
        dataIndex: 'total_transactions',
        key: 'total_transactions',
        align: 'left',
        className: 'ant_table',
        render: (text) => {
          return <FormattedNumber value={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'rewards'})),
        dataIndex: 'reward_per_blocks',
        key: 'reward_per_blocks',
        align: 'right',
        className: 'ant_table',
        render: (text) => {
          if (!text) return <span>-</span>
          return (
            <span><FormattedNumber value={text / ONE_UNIT}/>&nbsp;{CHAIN_SYMBOL}</span>
          )
        }
      },
      // {
      //   title: upperFirst(intl.formatMessage({id: 'bytes'})),
      //   dataIndex: 'size',
      //   key: 'size',
      //   align: 'right',
      //   className: 'ant_table',
      //   render: (text) => {
      //     return <FormattedNumber value={text}/>
      //   }
      // },
    ];
  }

  render () {
    let { total, loading, blocks, emptyState: EmptyState = null} = this.state;
    let column = this.customizedColumn();
    let {intl} = this.props;
    let tableInfo = intl.formatMessage({id: 'view_total'}) + ' ' + total + ' ' + intl.formatMessage({id: 'record_unit'})

    if (!loading && blocks.length === 0) {
      if (!EmptyState) {
        return (
          <div className="p-3 text-center  no-data">{t('no_blocks_found')}</div>
        );
      }
      return <EmptyState/>;
    }

    return (
      <div className="token_black table_pos">
        {loading && <div className="loading-style"><TronLoader/></div>}
        {total ? <div className="table_pos_info d-none d-md-block" style={{left: 'auto'}}>{tableInfo}</div> : ''}
        <SmartTable bordered={true} loading={loading} column={column} data={blocks} total={total}
                    onPageChange={(page, pageSize) => {
                      this.load(page, pageSize)
                    }}/>
      </div>
    )
  }
}


export default injectIntl(Blocks)
