import React from 'react';
import {FormattedNumber, injectIntl} from 'react-intl';
import {Client} from '../../services/api';
import {AddressLink, BlockNumberLink, TransactionHashLink} from './Links';
import {tu} from '../../utils/i18n';
import {FormatNumberByDecimals} from '../../utils/number'
import TimeAgo from 'react-timeago';
import moment from 'moment';
import {Truncate} from './text';
import {withTimers} from '../../utils/timing';
import SmartTable from './SmartTable.js'
import {upperFirst} from 'lodash';
import {TronLoader} from './loaders';
import TotalInfo from './TableTotal';
import DateRange from './DateRange';
import {mapTokenData} from '../../utils/helper';
import {CHAIN_SYMBOL, ONE_UNIT} from '../../constants';

class Rewards extends React.Component {

  constructor(props) {
    super(props);
    this.state = {
      filter: {},
      dataList: [],
      page: 1,
      total: 0,
      rangeTotal: 0,
      pageSize: 20,
      showTotal: props.showTotal !== false,
      emptyState: props.emptyState,
      hideSmallCurrency: true,
      tokenName: '_'
    };
  }

  componentDidMount() {
    let {page, pageSize} = this.state;
    this.load(page, pageSize);
  }

  onChange = (page, pageSize) => {
    this.setState({
      page: page,
      pageSize: pageSize
    }, () => {
      this.load(page, pageSize);
    });
  };

  load = async (page, pageSize) => {
    let {filter} = this.props;
    this.setState({
      loading: true,
      page: page,
      pageSize: pageSize,
    });
    const result = await Client.getRewards({
      limit: pageSize,
      start: (page - 1) * pageSize,
      ...filter
    });

    this.setState({
      page,
      dataList: result.items,
      total: result.total,
      rangeTotal: result['range_total'],
      loading: false,
    });
  };

  customizedColumn = () => {
    let {intl} = this.props;
    return [
      {
        title: upperFirst(intl.formatMessage({id: 'block'})),
        dataIndex: 'block_number',
        key: 'block_number',
        className: 'ant_table',
        render: (text) => {
          return <BlockNumberLink number={text}/>
        }
      }, {
        title: upperFirst(intl.formatMessage({id: 'address'})),
        dataIndex: 'address',
        key: 'address',
        width: '40%',
        className: 'ant_table',
        render: (text) => {
          return <AddressLink address={text}/>
        }
      }, {
        title: upperFirst(intl.formatMessage({id: 'type'})),
        dataIndex: 'type',
        className: 'ant_table',
        render: (text) => {
          return <span>{text}</span>
        }
      }, {
        title: upperFirst(intl.formatMessage({id: 'amount'})),
        dataIndex: 'amount',
        key: 'amount',
        align: 'right',
        className: 'ant_table _text_nowrap',
        render: (text) => {
          return (
            <span><FormattedNumber value={text / ONE_UNIT}/>&nbsp;{CHAIN_SYMBOL}</span>
          )
        }
      }
    ];
  };

  render() {
    let {dataList, total, loading, rangeTotal} = this.state;
    let column = this.customizedColumn();
    let {intl} = this.props;

    let locale = {emptyText: intl.formatMessage({id: 'no_transfers'})}

    return (
      <div className="token_black table_pos">
        {loading && <div className="loading-style"><TronLoader/></div>}
        <div className="d-flex justify-content-between" style={{left: 'auto'}}>
          {total ? <TotalInfo total={total} rangeTotal={rangeTotal} typeText="rewards"
                              divClass="table_pos_info_addr"/> : ''}
        </div>
        {
          (!loading && dataList.length === 0) ?
            <div className="p-3 text-center no-data">{'No Rewards'}</div>
            :
            <SmartTable bordered={true} loading={loading} column={column} data={dataList} total={total} locale={locale}
                        onPageChange={(page, pageSize) => {
                          this.onChange(page, pageSize)
                        }}/>
        }
      </div>
    )
  }
}

export default withTimers(injectIntl(Rewards));
