import React, { Fragment } from 'react';
import { tu } from '../../../utils/i18n';
import { AddressLink } from '../../common/Links';
import SmartTable from '../../common/SmartTable.js'
import { FormattedNumber, injectIntl } from 'react-intl';
import { TronLoader } from '../../common/loaders';
import { upperFirst } from 'lodash';
import xhr from 'axios/index';
import { API_URL } from '../../../constants';
import { toastr } from 'react-redux-toastr'
import { isAddressValid } from 'mcashscan/src/utils/crypto';
import { trim } from 'lodash'
import { FormatNumberByDecimals } from '../../../utils/number'
import { QuestionMark } from '../../common/QuestionMark';
import { Client } from '../../../services/api';

class TokenHolders extends React.Component {

  constructor (props) {
    super(props);
    this.state = {
      search: '',
      filter: {},
      addresses: [],
      page: 0,
      total: 0,
      pageSize: 25,
    };
  }

  componentDidMount () {
    this.loadTokenHolders();
  }

  onChange = (page, pageSize) => {
    this.loadTokenHolders(page, pageSize);
  };

  loadTokenHolders = async (page = 1, pageSize = 20) => {
    let {filter} = this.props;
    this.setState({loading: true});
    const result = await Client.getToken20Holders(filter.token, {
      start: (page - 1) * pageSize,
      limit: pageSize
    });

    let addresses = result.items ? result.items : [];
    let total = result.total;
    let rangeTotal = result.range_total;

    if (Array.isArray(addresses)) {
      addresses.forEach((item, index) => {
        item.index = index + 1 + (page - 1) * pageSize;
      })
    }

    this.setState({
      page,
      addresses,
      total,
      rangeTotal,
      loading: false,
    });
  };

  customizedColumn = () => {
    let {intl, token} = this.props;
    return [
      {
        title: '#',
        dataIndex: 'index',
        key: 'index',
        width: '5%',
        align: 'center',
        className: 'ant_table',
      },
      {
        title: intl.formatMessage({id: 'address'}),
        dataIndex: 'address',
        key: 'address',
        render: (text) => {
          return <AddressLink address={text}/>
        }
      },
      {
        title: upperFirst(intl.formatMessage({id: 'quantity'})),
        dataIndex: 'balance',
        key: 'balance',
        width: '20%',
        align: 'right',
        className: 'ant_table',
        render: (text, record) => {
          // return <FormattedNumber value={Number(record.balance) / (Math.pow(10,token.decimals))}/>
          return <span>{FormatNumberByDecimals(record.balance, token.decimals)}</span>
        }
      },
      {
        title: intl.formatMessage({id: 'percentage'}),
        dataIndex: 'percentage',
        key: 'percentage',
        width: '18%',
        align: 'right',
        className: 'ant_table',
        render: (text, record) => {
          return (
            <div>
              <FormattedNumber value={record.percentage} maximumFractionDigits={6}/>&nbsp;%
            </div>
          );
        }
      }
    ];
  }

  doSearch = async () => {
    let {intl, filter} = this.props;
    let {search} = this.state;

    if (isAddressValid(search)) {
      const result = await Client.getToken20Holders(filter.token, {
        address: search
      });
      let addresses = result.items ? result.items : [];
      addresses[0].index = 1
      this.setState({
        addresses,
        total: 1,
        search: ''
      });
    } else {
      toastr.warning(intl.formatMessage({id: 'warning'}), intl.formatMessage({id: 'search_TRC20_error'}));
      this.setState({
        search: ''
      });
    }
  };

  render () {
    let {addresses, total, rangeTotal, loading, search} = this.state;
    if (total === 0) {
      addresses = [];
    }
    let {intl} = this.props
    let column = this.customizedColumn();
    // let tableInfo = intl.formatMessage({id: 'a_totle'}) + ' ' + total + ' ' + intl.formatMessage({id: 'hold_addr'});
    let tableInfoTip = intl.formatMessage({id: 'table_info_holders_tip1'}) + ' ' + rangeTotal + ' ' + intl.formatMessage({id: 'table_info_holders_tip2'});
    if (!loading && addresses.length === 0) {
      return (
        <div className="p-3 text-center no-data">{tu('no_holders_found')}</div>
      );
    }
    return (
      <Fragment>
        {loading && <div className="loading-style" style={{marginTop: '-20px'}}><TronLoader/></div>}
        <div className="row transfers">
          <div className="col-md-12 table_pos">
            <div className="nav-searchbar" style={styles.searchBox}>
              <div className="token20-input-group input-group">
                <div className="token20-search">
                  <input type="text"
                         className="form-control p-2 bg-white border-0 box-shadow-none"
                         value={search}
                         onChange={ev => this.setState({search: trim(ev.target.value)})}
                         placeholder={intl.formatMessage({id: 'search_TRC20'})}/>
                  <div className="input-group-append">
                    <button className="btn box-shadow-none" onClick={this.doSearch}>
                      <i className="fa fa-search"/>
                    </button>
                  </div>
                </div>
              </div>
            </div>
            <div style={styles.tablePosInfo}>
              {total ? <div className="table_pos_info d-none d-md-block" style={{left: 'auto'}}>
                <div>{tu('view_total')} {rangeTotal} {tu('hold_addr')}

                  {rangeTotal >= 10000 ? <QuestionMark placement="top" info={tableInfoTip}/> : ''}
                  <br/>
                  {rangeTotal >= 10000 ? <span>({tu('table_info_big')})</span> : ''}
                </div>
              </div> : ''}
            </div>
            {/*{total?<div className="table_pos_info d-none d-md-block">{tableInfo}</div>: ''}*/}
            <div style={styles.table}>
              <SmartTable border={false} loading={loading} column={column} data={addresses} total={total}
                          onPageChange={(page, pageSize) => {
                            this.loadTokenHolders(page, pageSize)
                          }}/>
            </div>
          </div>
        </div>
      </Fragment>
    )
  }

}

const styles = {
  searchBox: {
    background: '#fff',
    paddingTop: 10,
  },
  tablePosInfo: {
    paddingLeft: 40
  }
};

export default injectIntl(TokenHolders);
