import {BigNumber} from 'bignumber.js'
import {CHAIN_DECIMAL} from '../constants'

export function serializeQuery(params, prefix) {
  const query = Object.keys(params).map((key) => {
    const value = params[key]

    if (params.constructor === Array) {
      key = `${prefix}[]`
    } else {
      if (params.constructor === Object) {
        key = (prefix ? `${prefix}[${key}]` : key)
      }
    }

    return value === 'object' ? this.serializeQuery(value, key) : `${key}=${encodeURIComponent(value)}`
  })
  return [].concat.apply([], query).join('&')
}

export function fromAmount(amount, decimals = CHAIN_DECIMAL) {
  // return amount / Math.pow(10, CHAIN_DECIMAL)
  return amount ? new BigNumber(amount).div(Math.pow(10, decimals)).toFixed() : '0'
}

export function toAmount(value, decimals = CHAIN_DECIMAL) {
  return value ? value * Math.pow(10, decimals) : 0
  // return value ? new BigNumber(value).times(Math.pow(10, CHAIN_DECIMAL)).toFixed() : '0'
}

export function isTriggerSmartContract(contractType) {
  return contractType === 31
}

// --- get value in object ---
// extends object-path
let options = {};

function getKey(key) {
  let intKey = parseInt(key);
  if (intKey.toString() === key) {
    return intKey;
  }
  return key;
}

function hasOwnProperty(obj, prop) {
  if (obj === null) {
    return false
  }
  //to handle objects with null prototypes (too edge case?)
  return Object.prototype.hasOwnProperty.call(obj, prop)
}

function hasShallowProperty(obj, prop) {
  return (options.includeInheritedProps || (typeof prop === 'number' && Array.isArray(obj)) || hasOwnProperty(obj, prop))
}

function getShallowProperty(obj, prop) {
  if (hasShallowProperty(obj, prop)) {
    return obj[prop];
  }
}

export function gets(obj, path, defaultValue = null) {
  if (typeof path === 'number') {
    path = [path];
  }
  if (!path || path.length === 0) {
    return obj;
  }
  if (obj === null) {
    return defaultValue;
  }
  if (typeof path === 'string') {
    return gets(obj, path.split('.'), defaultValue);
  }

  let currentPath = getKey(path[0]);
  let nextObj = getShallowProperty(obj, currentPath);
  if (nextObj === void 0) {
    return defaultValue;
  }

  if (path.length === 1) {
    return nextObj;
  }

  return gets(obj[currentPath], path.slice(1), defaultValue);
}

export function mapContractTokenData(result) {
  return mapTokenData(result.items, result['token_map'], 'contract_address')
}

export function mapAssetTokenData(result) {
  return mapTokenData(result.items, result['token_map'], 'asset_id')
}

export function mapTokenObject(obj, tokens, key = 'contract_address') {
  if (typeof obj[key] !== 'undefined') {
    return {
      ...obj,
      token: tokens[obj[key]],
    }
  }
  return {}
}

export function mapTokenData(items, tokens, key = 'contract_address') {
  if (items) {
    if (Array.isArray(items) && tokens) {
      return items.map(item => ({
        ...item,
        token: typeof item[key] !== 'undefined' ? tokens[item[key]] : {}
      }))
    }
  }
  return []
}

export function validateAddress(address) {
  if (!address) return true
  const addressesBlackList = ['GENESIS']
  return addressesBlackList.indexOf(address) === -1
}

export function isHex(c) {
  return ('0123456789ABCDEFabcdef'.indexOf(c) > -1);
}

export function numberFormat(number, decimals, decPoint, thousandsSep = '', trailingZeros = true) {
  if (typeof number === 'undefined') return
  // Strip all characters but numerical ones.
  number = (number + '').replace(/[^0-9+\-Ee.]/g, '')
  let n = !isFinite(+number) ? 0 : +number
  let prec = !isFinite(+decimals) ? 0 : Math.abs(decimals)
  let sep = (typeof thousandsSep === 'undefined') ? ',' : thousandsSep
  let dec = (typeof decPoint === 'undefined') ? '.' : decPoint
  let s = ''
  let toFixedFix = function (n, prec) {
    let k = Math.pow(10, prec)
    return '' + Math.round(n * k) / k
  }
  // Fix for IE parseFloat(0.55).toFixed(0) = 0;
  s = (prec ? toFixedFix(n, prec) : '' + Math.round(n)).split('.')
  if (s[0].length > 3) {
    s[0] = s[0].replace(/\B(?=(?:\d{3})+(?!\d))/g, sep)
  }
  if ((s[1] || '').length < prec) {
    s[1] = s[1] || ''
    if (trailingZeros) {
      // 1.123 with decimals = 5 => 1.12300
      s[1] += new Array(prec - s[1].length + 1).join('0')
    }
  }
  return s[1] ? s.join(dec) : s[0]
}

export function showAmount(amount, decimals = CHAIN_DECIMAL) {
  if (isNaN(amount)) {
    return '-'
  }
  const n = new BigNumber(amount)
  const d = new BigNumber(1).div(Math.pow(10, decimals))
  if (n.gt(0) && n.lt(d)) {
    return '<' + d.toFixed()
  }
  return n.toFixed()
}

export function getAccountBandwidth(accountData) {
  if (!accountData) {
    return {}
  }
  const {account_resource_detail = {}} = accountData
  const {bandwidth_used = 0, free_bandwidth_used = 0, bandwidth_limit = 0, free_bandwidth_limit = 0} = account_resource_detail
  const totalBandwidthUsed = bandwidth_used + free_bandwidth_used
  const totalBandwidthLimit = bandwidth_limit + free_bandwidth_limit
  const bandwidthRemaining = bandwidth_limit - bandwidth_used
  const freeBandwidthRemaining = free_bandwidth_limit - free_bandwidth_used
  const totalBandwidthRemaining = totalBandwidthLimit - totalBandwidthUsed
  const bandWidthPercentage = (totalBandwidthUsed / totalBandwidthLimit) * 100
  const availableBandWidthPercentage = 100 - bandWidthPercentage
  const freeBandWidthPercentage = (free_bandwidth_used / free_bandwidth_limit) * 100
  return {
    bandwidthUsed: bandwidth_used,
    freeBandwidthUsed: free_bandwidth_used,
    bandwidthLimit: bandwidth_limit,
    freeBandwidthLimit: free_bandwidth_limit,
    totalBandwidthUsed,
    totalBandwidthLimit,
    bandwidthRemaining,
    freeBandwidthRemaining,
    totalBandwidthRemaining,
    bandWidthPercentage,
    availableBandWidthPercentage,
    freeBandWidthPercentage
  }
}

export function getAccountEnergy(accountData) {
  if (!accountData) {
    return {}
  }
  const {account_resource_detail = {}} = accountData
  const {energy_limit = 0, energy_used = 0} = account_resource_detail
  const energyUsed = energy_used
  const energyLimit = energy_limit
  const energyRemaining = energyLimit - energyUsed
  const energyPercentage = (energyUsed / energyLimit) * 100
  const availableEnergyPercentage = 100 - energyPercentage
  return {
    energyUsed,
    energyLimit,
    energyRemaining,
    energyPercentage,
    availableEnergyPercentage
  }
}

export function prettyErrorMessage(message) {
  const errorMessageRegex = new RegExp('class [a-zA-Z._]+ :');
  return message.replace(errorMessageRegex, '').trim();
}
